#!/usr/bin/python2.2

import gtk
import gtk.glade
import string
import sambaToken
import mainWindow
import libuser

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = 'redhat-config-samba'
translate.textdomain (domain)
gtk.glade.bindtextdomain(domain)

class BasicPreferencesWin:

    def __init__(self, parent, xml, samba_data, samba_backend, main_window):
        self.ParentClass = parent
        self.samba_data = samba_data
        self.samba_backend = samba_backend
        self.samba_file_list = samba_data.getList()

        self.basic_notebook = xml.get_widget("basic_notebook")
        self.basic_preferences_win = xml.get_widget("basic_preferences_win")
        self.basic_preferences_win.set_modal(gtk.TRUE)
        self.basic_preferences_win.set_transient_for(main_window)
        self.basic_preferences_win.connect("delete-event", self.onBasicCancelButtonClicked)
        self.basic_preferences_win.set_icon(mainWindow.iconPixbuf)
        self.workgroup_entry = xml.get_widget("workgroup_entry")
        self.server_entry = xml.get_widget("server_entry")
        self.auth_server_entry = xml.get_widget("auth_server_entry")

        self.auth_option_menu = xml.get_widget("auth_option_menu")
        self.auth_menu = gtk.Menu()
        label = (_("ADS"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "ADS")
        self.auth_menu.append(item)
        label = (_("Domain"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "DOMAIN")
        self.auth_menu.append(item)
        label = (_("Server"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "SERVER")
        self.auth_menu.append(item)
        label = (_("Share"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "SHARE")
        self.auth_menu.append(item)
        label = (_("User"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "USER")
        self.auth_menu.append(item)
        self.auth_option_menu.set_menu(self.auth_menu)

        self.encrypt_option_menu = xml.get_widget("encrypt_option_menu")
        self.encrypt_menu = gtk.Menu()
        label = (_("Yes"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "yes")
        self.encrypt_menu.append(item)
        label = (_("No"))
        item = gtk.MenuItem(label)
        item.set_data("NAME", "no")
        self.encrypt_menu.append(item)
        self.encrypt_option_menu.set_menu(self.encrypt_menu)

        self.guest_option_menu = xml.get_widget("guest_option_menu")
        self.guest_menu = gtk.Menu()
        self.admin = libuser.admin()
        self.users = self.admin.enumerateUsers()
        self.users.sort()
        self.users.insert(0, _("No guest account"))
        for user in self.users:
            item = gtk.MenuItem(user)
            item.set_data("NAME", user)
            self.guest_menu.append(item)
        self.guest_option_menu.set_menu(self.guest_menu)

        xml.signal_connect("on_basic_cancel_button_clicked", self.onBasicCancelButtonClicked)
        xml.signal_connect("on_basic_ok_button_clicked", self.onBasicOkButtonClicked)
        self.auth_option_menu.connect("changed", self.authMenuChanged)

    def showWindow(self):
        self.readFile()
        self.reset()
        self.basic_preferences_win.show_all()

    def readFile(self):
        global_found = None
        guest_account = None
        #Set auth_option_menu default to "User" since that is the default value for smb.conf
        self.auth_option_menu.set_history(3)
        #Set the encrypt_option_menu default to "Yes"
        self.encrypt_option_menu.set_history(0)
        
        for line in self.samba_file_list:
            parent_tag = line.getParent()

            if line.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL and parent_tag == "[global]":
                if string.lower(line.keyname) == "workgroup":
                    self.workgroup_entry.set_text(string.lower(line.keyval))
                elif string.lower(line.keyname) == "server string":
                    self.server_entry.set_text(string.lower(line.keyval))
                elif string.lower(line.keyname) == "security":
                    if string.lower(line.keyval) == "ads":
                        self.auth_option_menu.set_history(0)                        
                    elif string.lower(line.keyval) == "domain":
                        self.auth_option_menu.set_history(1)
                    elif string.lower(line.keyval) == "server":
                        self.auth_option_menu.set_history(2)
                    elif string.lower(line.keyval) == "share":
                        self.auth_option_menu.set_history(3)
                    elif string.lower(line.keyval) == "user":
                        self.auth_option_menu.set_history(4)
                elif string.lower(line.keyname) == "password server":
                    if line.keyval != "None":
                        self.auth_server_entry.set_text(line.keyval)

                elif string.lower(line.keyname) == "encrypt passwords":
                    if string.lower(line.keyval) == "yes":
                        self.encrypt_option_menu.set_history(0)
                    elif string.lower(line.keyval) == "no":
                        self.encrypt_option_menu.set_history(1)
                
                elif line.keyname == "guest ok":
                    if line.keyval == "no":
                        guest_account = None
                        self.guest_option_menu.set_history(0)
                    else:
                        #If guest accounts are enabled, let's assume the default is 'nobody'
                        guest_account = "nobody"
                    
                elif line.keyname == "guest account":
                    #if the default isn't nobody, we'll catch it here
                    guest_account = line.keyval

        #if guest accounts are enabled, lets set the menu to the guest user
        if guest_account:
            count = 0
            found = 0
            for user in self.users:
                if guest_account == user:
                    found = count
                count = count + 1
            self.guest_option_menu.set_history(found)            
                            
    def reset(self):
        self.basic_notebook.set_current_page(0)

    def onBasicOkButtonClicked(self, *args):
        #Check to see if workgroup is specified
        if not self.checkForWorkgroup(self.workgroup_entry.get_text()):
            return
        else:
            self.ParentClass.changeValue("workgroup", self.workgroup_entry.get_text(), "[global]")
            self.ParentClass.changeValue("server string", self.server_entry.get_text(), "[global]")

            auth_type = self.auth_menu.get_active().get_data("NAME")
            if auth_type == "USER":
                self.ParentClass.changeValue("username map", "/etc/samba/smbusers", "[global]")
                self.ParentClass.changeValue("password server", "None", "[global]")

            elif auth_type == "SHARE":
                self.ParentClass.changeValue("username map", "None", "[global]")                
                self.ParentClass.changeValue("password server", "None", "[global]")

            else:
                self.ParentClass.changeValue("username map", "None", "[global]")
                
                if auth_type == "SERVER" or auth_type == "DOMAIN" or auth_type == "ADS":
                    #If they've specified SERVER or DOMAIN, require a password server
                    auth_server = string.strip(self.auth_server_entry.get_text())
                    
                    if auth_server == "":
                        dlg = gtk.MessageDialog(self.basic_preferences_win, 0, gtk.MESSAGE_WARNING,
                                                gtk.BUTTONS_OK,
                                                (_("You must specify a password server when using 'ads', 'domain' "
                                                   "or 'server' authentication.")))
                        dlg.set_position(gtk.WIN_POS_CENTER)
                        dlg.set_modal(gtk.TRUE)
                        dlg.set_transient_for(self.basic_preferences_win)
                        dlg.set_icon(mainWindow.iconPixbuf)            
                        dlg.run()
                        dlg.destroy()
                        self.auth_server_entry.grab_focus()
                        return 
                    else:
                        self.ParentClass.changeValue("password server", auth_server, "[global]")
                    
            self.ParentClass.changeValue("security", self.auth_menu.get_active().get_data("NAME"),
                                         "[global]")



            self.ParentClass.changeValue("encrypt passwords", self.encrypt_menu.get_active().get_data("NAME"),
                                         "[global]")        


            if self.guest_option_menu.get_history() == 0:
                self.ParentClass.changeValue("guest ok", "no", "[global]")
                self.ParentClass.changeValue("guest account", "nobody", "[global]")
            else:
                self.ParentClass.changeValue("guest ok", "yes", "[global]")
                self.ParentClass.changeValue("guest account", self.guest_menu.get_active().get_data("NAME"), "[global]")

            self.basic_preferences_win.hide()
            
            #Let's go ahead and restart the service
            self.samba_data.writeFile()
            self.samba_backend.restartSamba()

    def onBasicCancelButtonClicked(self, *args):
        self.basic_preferences_win.hide()
        return gtk.TRUE

    def checkForWorkgroup(self, workgroup):
        if string.strip(workgroup) == "":
            dlg = gtk.MessageDialog(self.basic_preferences_win, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("You must specify a workgroup.")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.basic_preferences_win)
            dlg.set_icon(mainWindow.iconPixbuf)            
            dlg.run()
            dlg.destroy()
            return 0
        return 1

    def authMenuChanged(self, *args):
        type = self.auth_menu.get_active().get_data("NAME")

        #Let's enable/disable the auth_server_entry
        if type == "SERVER" or type == "DOMAIN" or type == "ADS":
            #allow password server if using 'server' or 'domain'
            self.auth_server_entry.set_sensitive(gtk.TRUE)
        else:
            self.auth_server_entry.set_sensitive(gtk.FALSE)            

        #Must force encrypted passwords with 'domain'
        if type == "DOMAIN":
            self.encrypt_option_menu.set_history(0)
            self.encrypt_option_menu.set_sensitive(gtk.FALSE)
        else:
            self.encrypt_option_menu.set_sensitive(gtk.TRUE)
