#!/usr/bin/python

import string
import os
import gtk
import sambaToken
import mainWindow
from sambaDefaults import global_keys
from sambaDefaults import section_keys

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = 'redhat-config-samba'
translate.textdomain (domain)

class SambaParser:
    def __init__(self, parent):
        self.ParentClass = parent

        self.samba_file_list = []
        self.parseFile()

    def parseFile(self):
        path = "/etc/samba/smb.conf"
        lines = []

        if os.access(path, os.F_OK) == 0:
            #If path doesn't exist, just return.  We'll create a new one later
            return
        
        if os.access(path, os.R_OK) == 1:
            #Check to see if we can read from the file
            fd = open(path, 'r')
            lines = fd.readlines()
            fd.close()
        else:
            #The file exists, but we can't read it.  Not much we can do now but quit.
            dlg = gtk.MessageDialog(self.ParentClass.main_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("Cannot read %s.  Program will now exit." % path)))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_icon(mainWindow.iconPixbuf)
            dlg.run()
            dlg.destroy()
            gtk.mainquit()
            raise RuntimeError, (_("Cannot read %s.  Program will now exit." % path))
        
        self.parent = None

        if lines:
            for line in lines:
                token = self.createToken(line) 
                if token:
                    #If the token is valid, then add it
                    self.samba_file_list.append(token)

    def createToken(self, line, parent=None):
        if parent:
            self.parent = parent

        stripped_line = string.strip(line)
        if stripped_line != "":
            tmp = tuple(stripped_line)
        else:
            if self.parent:
                token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_BLANKLINE, (stripped_line), self.parent)
            else:
                token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_BLANKLINE, (stripped_line))
#            self.samba_file_list.append(token)
            return token

        if tmp:
            if tmp[0] == "#" or tmp[0] == ";":
                #The line begins with a "#"
                if self.parent:
                    token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_STRING, (stripped_line), self.parent)
                else:
                    token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_STRING, (stripped_line))
#                self.samba_file_list.append(token)
                return token

            elif tmp[0] == "[" and tmp[-1] == "]":
                #The line contains a section header
                token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_SECTION_HEADER, (stripped_line))
#                self.samba_file_list.append(token)
                self.parent = stripped_line
                return token

            else:
                #The line isn't a section header and is probably a key/value line
                comment_token = None

                #See if there are any comments at the end of the line
                try:
                    data, comment = string.split(stripped_line, "#")
                    comment_token = comment
                    stripped_line = data
                except:
                    #There are no comments in the line
                    data = line
                    pass

                try:
                    #See if there's only one "=" in the line
                    name, value = string.split(stripped_line, "=")
                    name = string.strip(name)
                    value = string.strip(value)
                except:
                    #Nope, there are more than one "=" in the line, so do some reordering of the tokens
                    tokens = string.split(data, "=")
                    name = tokens[0]
                    name = string.strip(name)
                    tmp_list = []
                    for token in tokens[1:]:
                        tmp_token = string.strip(token)
                        tmp_list.append(tmp_token)
                    value = string.joinfields(tmp_list, '=')

                if comment_token:
                    token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL, (name, value), self.parent, comment_token)
                    return token
                else:
                    if not self.isDuplicateKey(name, self.parent):
                        token = sambaToken.SambaToken(sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL, (name, value), self.parent)
                        return token
                    else:
                        #We've found a duplicate key, so return none.  This keeps duplicates from being
                        #added to the list
                        return None

    def writeFile(self):
        path = "/etc/samba/smb.conf"
        if os.access(path, os.W_OK) == 1:
            fd = open(path, 'w')
        else:
            dlg = gtk.MessageDialog(self.ParentClass.main_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("Cannot write to %s.  Program will now exit." % path)))
            dlg.set_modal(gtk.TRUE)
            dlg.set_icon(mainWindow.iconPixbuf)
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.run()
            dlg.destroy()
            gtk.mainquit()
            raise RuntimeError, (_("Cannot write to %s.  Program will now exit." % path))
        

        if fd:
            for object in self.samba_file_list:
                line = object.getData()
                if line:
                    fd.write(line)
            fd.close()

    def getList(self):
        return self.samba_file_list

    def getShareHeaders(self):
        header_list = []
        for object in self.samba_file_list:
            if object.type == sambaToken.SambaToken.SAMBA_TOKEN_SECTION_HEADER:
                if object.value != "[global]" and object.value != "[printers]" and object.value != "[homes]":
                    header_list.append(object.value)
        return header_list

    def isDuplicateKey(self, name, parent):
        found = None
        for object in self.samba_file_list:
            if object.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL:
                if parent == object.getParent():
                    keyname, keyvalue = object.value
                    if keyname == name:
                        return 1
