#!/usr/bin/python2.2

import gtk
import string
import mainWindow
import sambaToken
import sambaUserData
import os
import gobject

##
## I18N
##
from rhpl.translate import _, N_
import rhpl.translate as translate
domain = 'redhat-config-samba'
translate.textdomain (domain)
gtk.glade.bindtextdomain(domain)

class ShareWindow:
    def __init__(self, parent, xml, samba_data, samba_user_data, samba_backend, main_window):
        self.ParentClass = parent
        self.samba_data = samba_data
        self.samba_user_data = samba_user_data
        self.samba_backend = samba_backend
        self.samba_file_list = samba_data.getList()
        
        self.share_window = xml.get_widget("share_win")
        self.share_window.set_modal(gtk.TRUE)
        self.share_window.set_transient_for(main_window)
        self.share_window.connect("delete-event", self.onCancelButtonClicked)
        self.share_window.set_position(gtk.WIN_POS_CENTER)
        self.share_window.set_icon(mainWindow.iconPixbuf)
        self.share_notebook = xml.get_widget("share_notebook")
        self.dir_entry = xml.get_widget("share_dir_entry")
        self.description_entry = xml.get_widget("description_entry")
        self.read_radio = xml.get_widget("share_read_radio")
        self.write_radio = xml.get_widget("share_write_radio")

        self.user_access_radio = xml.get_widget("user_access_radio")
        self.guest_access_radio = xml.get_widget("guest_access_radio")
        self.user_access_radio.connect("toggled", self.userRadioToggled)
        
        
        self.ok_button = xml.get_widget("share_ok_button")

        xml.signal_connect("on_share_cancel_button_clicked", self.onCancelButtonClicked)
        xml.signal_connect("on_share_browse_button_clicked", self.onBrowseButtonClicked)

        self.valid_users_treeview = xml.get_widget("valid_users_treeview")
        self.browsable_checkbutton = xml.get_widget("browsable_checkbutton")
        self.create_mode_label = xml.get_widget("create_mode_label")
        self.dir_mode_label = xml.get_widget("dir_mode_label")
        self.create_mode_button = xml.get_widget("create_mode_button")
        self.dir_mode_button = xml.get_widget("dir_mode_button")        

        self.valid_users_store = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING)
        self.valid_users_treeview.set_model(self.valid_users_store)

        self.checkbox = gtk.CellRendererToggle()
        col = gtk.TreeViewColumn('', self.checkbox, active = 0)
        col.set_fixed_width(20)
        col.set_clickable(gtk.TRUE)
        self.checkbox.connect("toggled", self.userToggled)
        self.valid_users_treeview.append_column(col)

        col = gtk.TreeViewColumn("", gtk.CellRendererText(), text=1)
        self.valid_users_treeview.append_column(col)

    def populateUserStore(self):
        userList = self.samba_user_data.getPasswdFile()

        if userList == None:
            return

        userList.sort()
        for line in userList:
            iter = self.valid_users_store.append()
            tokens = string.split(line, ':')
            self.valid_users_store.set_value(iter, 0, gtk.FALSE)
            self.valid_users_store.set_value(iter, 1, tokens[0])

    def populateUserStoreOnEdit(self, currentUserList, invalidUsers):
        userList = self.samba_user_data.getPasswdFile()

        if userList == None:
            return

        userList.sort()
        for line in userList:
            iter = self.valid_users_store.append()
            tokens = string.split(line, ':')

            if self.guest_access_radio.get_active() == gtk.TRUE:
                self.valid_users_store.set_value(iter, 0, gtk.FALSE)
                self.valid_users_store.set_value(iter, 1, tokens[0])
                continue
                
            if invalidUsers == "%S":
                #Make all users unselected
                self.valid_users_store.set_value(iter, 0, gtk.FALSE)
                self.valid_users_store.set_value(iter, 1, tokens[0])
            else:
                #invalidUsers is not "%S"
                if currentUserList == ['None'] or currentUserList == []:
                    #If no users are specified, assume all are allowed
                    self.valid_users_store.set_value(iter, 0, gtk.TRUE)
                    self.valid_users_store.set_value(iter, 1, tokens[0])
                else:
                    #Let's see which users are allowed
                    if tokens[0] in currentUserList:
                        self.valid_users_store.set_value(iter, 0, gtk.TRUE)
                        self.valid_users_store.set_value(iter, 1, tokens[0])
                    else:
                        self.valid_users_store.set_value(iter, 0, gtk.FALSE)
                        self.valid_users_store.set_value(iter, 1, tokens[0])

    def userToggled(self, data, row):
        iter = self.valid_users_store.get_iter((int(row),))
        val = self.valid_users_store.get_value(iter, 0)
        self.valid_users_store.set_value(iter, 0 , not val)

    def showNewWindow(self):
        self.share_window.set_title(_("Create Samba Share"))
        self.ok_button_handler = self.ok_button.connect("clicked", self.onOkAddButtonClicked)
        self.dir_entry.grab_focus()
        self.populateUserStore()
        self.share_window.show_all()

    def showEditWindow(self, iter, parent):
        self.edit_iter = iter
        self.share_window.set_title(_("Edit Samba Share"))
        self.ok_button_handler = self.ok_button.connect("clicked", self.onOkEditButtonClicked)
        self.read_radio.set_active(gtk.TRUE)
        
        userList = []
        invalidUsers = None

        for line in self.samba_file_list:
            if line.type == sambaToken.SambaToken.SAMBA_TOKEN_KEYVAL and line.parent == parent:
                if line.keyname == "path":
                    self.dir_entry.set_text(line.keyval)
                elif line.keyname == "comment":
                    if line.keyval != "None":
                        self.description_entry.set_text(line.keyval)
                elif line.keyname == "writeable":
                    if string.lower(line.keyval) == "yes":
                        self.write_radio.set_active(gtk.TRUE)
                    else:
                        self.read_radio.set_active(gtk.TRUE)
                        
                elif string.lower(line.keyname) == "guest ok":
                    if string.lower(line.keyval) == "yes":
                        self.guest_access_radio.set_active(gtk.TRUE)
                    else:
                        self.user_access_radio.set_active(gtk.TRUE)

                if line.keyname == "valid users":
                    if line.keyval:
                        list = string.split(line.keyval, ",")
                        for item in list:
                            userList.append(string.strip(item))

                if line.keyname == "invalid users":
                    if line.keyval and line.keyval == "%S":
                        invalidUsers = line.keyval

        self.populateUserStoreOnEdit(userList, invalidUsers)
        self.share_window.show_all()

    def reset(self):
        self.share_notebook.set_current_page(0)
        self.dir_entry.set_text("")
        self.description_entry.set_text("")
        self.valid_users_store.clear()
        self.read_radio.set_active(gtk.TRUE)
        self.ok_button.disconnect(self.ok_button_handler)
        self.share_window.hide()

    def checkDirectoryValidity(self, dir):
        if string.strip(dir) == "":
            dlg = gtk.MessageDialog(self.share_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("You must specify a directory to share.  \n\n"
                                       "Click \"OK\" to continue.")))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.share_window)
            dlg.set_icon(mainWindow.iconPixbuf)            
            dlg.run()
            dlg.destroy()
            return 0

        try:
            os.stat(dir)
        except:
            dlg = gtk.MessageDialog(self.share_window, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("The directory \"%s\" does not exist.  Please specify "
                                       "an existing directory. \n\n"
                                       "Click \"OK\" to continue." % dir)))
            dlg.set_position(gtk.WIN_POS_CENTER)
            dlg.set_modal(gtk.TRUE)
            dlg.set_transient_for(self.share_window)
            dlg.set_icon(mainWindow.iconPixbuf)
            dlg.run()
            dlg.destroy()
            return 0                  

        return 1

    #################Event Handlers######################
    def onOkAddButtonClicked(self, *args):
        # Get the path from the widget
        path = self.dir_entry.get_text()
        # Strip off any whitespace
        path = string.strip(path)

        #Check to see if directory exists
        if not self.checkDirectoryValidity(path):
            return

        if path == "/":
            #Sharing the root is a special case
            dir = "/"
            dir_header = "[root directory]"

        else:
            #Check to see if the path ends in a "/"  If it does, strip it off
            if path[-1:] == "/":
                path = path[:-1]

            #If there are any /'s or \'s in the path, split by them
            if '/' in path:
                tokens = string.split(path, "/")
            
            #The last item in the token list is the directory name that we want
            dir = tokens[len(tokens)-1]
        
            #Build the section header name by bookending dir with '[' and ']'
            dir_header = "[" + dir + "]"

        #If there's already a section header with this name, then start adding number to it
        #until it's a unique name
        if dir_header in self.samba_data.getShareHeaders():
            count = 1
            while dir_header in self.samba_data.getShareHeaders():
                dir_header = "[" + dir + "-" + str(count) + "]"
                count = count + 1

        #Ok, things are good now.  Start adding to the share_store
        iter = self.ParentClass.share_store.append()
        self.ParentClass.share_store.set_value(iter, 0, path)

        #create a blank line token
        if self.samba_data.getList()[-1].getData() != "\n":
            token = self.samba_data.createToken("")
            self.samba_file_list.append(token)
        
        #create token for the section header
        token = self.samba_data.createToken(dir_header)
        self.samba_file_list.append(token)

        #create token for the description if it exists
        description = string.strip(self.description_entry.get_text())

        if description != "":
            while description[-1] == "\\" or description[-1] == " ":
                #If description ends in a backslash, chop it off b/c it confuses Windows
                description = description[:-1]

            token = self.samba_data.createToken("comment = " + description, dir_header)
            self.samba_file_list.append(token)
            self.ParentClass.share_store.set_value(iter, 2, description)

        #create token for the path
        token = self.samba_data.createToken("path = " + path, dir_header)
        self.samba_file_list.append(token)

        #create token for permissions        
        if self.read_radio.get_active() == gtk.TRUE:
            self.ParentClass.share_store.set_value(iter, 1, (_("Read Only")))
        else:
            token = self.samba_data.createToken("read only = no", dir_header)
            self.samba_file_list.append(token)
            self.ParentClass.share_store.set_value(iter, 1, (_("Read / Write")))

        if self.guest_access_radio.get_active() == gtk.TRUE:
            #create token for guest access
            token = self.samba_data.createToken("guest ok = yes", dir_header)
            self.samba_file_list.append(token)
        else:
            #they want user level access
            all_users = []
            selected_users = []

            user_iter = self.valid_users_store.get_iter_first()
            while user_iter:
                #Crawl through the list and see which users are selected
                all_users.append(self.valid_users_store.get_value(user_iter, 1))
                if self.valid_users_store.get_value(user_iter, 0) == gtk.TRUE:
                    selected_users.append(self.valid_users_store.get_value(user_iter, 1))
                user_iter = self.valid_users_store.iter_next(user_iter)

            if selected_users == []:
                #No users are selected.  Let's deny all users then
                token = self.samba_data.createToken("invalid users = %S", dir_header)
                self.samba_file_list.append(token)

            elif selected_users != [] and all_users != selected_users:
                #They have selected a subset of all samba users.
                users = string.join(selected_users, ", ")
                token = self.samba_data.createToken("valid users = %s" % users, dir_header)
                self.samba_file_list.append(token)
                
        self.ParentClass.share_store.set_value(iter, 3, dir_header)            
        self.ParentClass.properties_button.set_sensitive(gtk.FALSE)
        self.ParentClass.delete_button.set_sensitive(gtk.FALSE)
        self.ParentClass.share_view.get_selection().unselect_all()
        self.reset()

        #Let's go ahead and restart the service.
        self.samba_data.writeFile()
        self.samba_backend.restartSamba()

    def onOkEditButtonClicked(self, *args):
        parent = self.ParentClass.share_store.get_value(self.edit_iter, 3)

        dir = self.dir_entry.get_text()
        dir = string.strip(dir)

        #Check to see if directory exists
        if not self.checkDirectoryValidity(dir):
            return

        self.ParentClass.share_store.set_value(self.edit_iter, 0, dir)
        self.ParentClass.changeValue("path", dir, parent)

        if self.read_radio.get_active() == gtk.TRUE:
            self.ParentClass.share_store.set_value(self.edit_iter, 1, (_("Read Only")))
            self.ParentClass.changeValue("writeable", "no", parent)
        else:
            self.ParentClass.share_store.set_value(self.edit_iter, 1, (_("Read / Write")))
            self.ParentClass.changeValue("writeable", "yes", parent)

        description = string.strip(self.description_entry.get_text())

        if description != "":
            while description[-1] == "\\" or description[-1] == " ":
                #If description ends in a backslash, chop it off b/c it confuses Windows
                description = description[:-1]

        if description == "":
            self.ParentClass.share_store.set_value(self.edit_iter, 2, description)
            self.ParentClass.changeValue("comment", "None", parent)
        else:
            self.ParentClass.share_store.set_value(self.edit_iter, 2, description)
            self.ParentClass.changeValue("comment", description, parent)

        #change the token for user access
        if self.guest_access_radio.get_active() == gtk.TRUE:
            self.ParentClass.changeValue("guest ok", "yes", parent)
            self.ParentClass.changeValue("valid users", "None", parent)
            self.ParentClass.changeValue("invalid users", "None", parent)            
        else:
            self.ParentClass.changeValue("guest ok", "no", parent)

            #they want user level access
            all_users = []
            selected_users = []

            user_iter = self.valid_users_store.get_iter_first()
            while user_iter:
                #Crawl through the list and see which users are selected
                all_users.append(self.valid_users_store.get_value(user_iter, 1))
                if self.valid_users_store.get_value(user_iter, 0) == gtk.TRUE:
                    selected_users.append(self.valid_users_store.get_value(user_iter, 1))
                user_iter = self.valid_users_store.iter_next(user_iter)

            if selected_users == []:
                #No users are selected.  Set 'valid users' to none and 'invalid users' to '%S'
                self.ParentClass.changeValue("valid users", "None", parent)
                self.ParentClass.changeValue("invalid users", "%S", parent)
            else:
                if all_users == selected_users:
                    #They want to allow all users.  Remove "invalid_users" and "valid users"
                    self.ParentClass.changeValue("valid users", "None", parent)
                    self.ParentClass.changeValue("invalid users", "None", parent)
                else:
                    #They want to allow a subset of all samba users.
                    users = string.join(selected_users, ", ")
                    self.ParentClass.changeValue("valid users", users, parent)
                    self.ParentClass.changeValue("invalid users", "None", parent)

        self.ParentClass.properties_button.set_sensitive(gtk.FALSE)
        self.ParentClass.delete_button.set_sensitive(gtk.FALSE)
        self.ParentClass.share_view.get_selection().unselect_all()
        self.reset()

        #Let's go ahead and restart the service.
        self.samba_data.writeFile()
        self.samba_backend.restartSamba()

    def onCancelButtonClicked(self, *args):
        self.reset()
        return gtk.TRUE

    def onBrowseButtonClicked(self, *args):
        fb = gtk.FileSelection(_("Select Directory"))
        fb.set_position(gtk.WIN_POS_CENTER)
        fb.set_icon(mainWindow.iconPixbuf)
        fb.show_all()
        result = fb.run()        
        fb.hide()

        if result == gtk.RESPONSE_OK:
            self.dir_entry.set_text(fb.get_filename())

    def userRadioToggled(self, *args):
        self.valid_users_treeview.set_sensitive(self.user_access_radio.get_active())
        

